@@ -167,8 +167,8 @@ XOpenDisplay (
 #if USE_XCB
 	if(!_XConnectXCB(dpy, display, &fullname, &iscreen)) {
 		dpy->display_name = fullname;
-		OutOfMemory(dpy, 0);
-		return 0;
+		OutOfMemory(dpy, NULL);
+		return NULL;
 	}
 #else /* !USE_XCB */
 	if ((dpy->trans_conn = _X11TransConnectDisplay (
@@ -790,6 +790,17 @@ _XBigReqHandler(
 
 void _XFreeDisplayStructure(Display *dpy)
 {
+	/* move all cookies in the EQ to the jar, then free them. */
+	if (dpy->qfree) {
+	    _XQEvent *qelt = dpy->qfree;
+	    while (qelt) {
+		if (_XIsEventCookie(dpy, &qelt->event))
+		    _XStoreEventCookie(dpy, &qelt->event);
+		qelt = qelt->next;
+	    }
+        }
+	if (dpy->cookiejar)
+	    _XFreeEventCookies(dpy);
 	while (dpy->ext_procs) {
 	    _XExtension *ext = dpy->ext_procs;
 	    dpy->ext_procs = ext->next;
